#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <assert.h>
#include <math.h>
#include <curl/curl.h>

#include <sys/stat.h>
#include <sys/types.h>

#include "httpforward.h"
#include "logger.h"


HttpForward::HttpForward( const char *_pURL, bool _usePost )
{
    log( LOG_INFO, "Creating HttpForward to %s, method %s", (int)_pURL, (int)(_usePost ? "Post" : "Get") );

    // _pURLformat commes from another context, so duplicate to be safe
    pURL = strdup( _pURL );
    usePost = _usePost;

    curl_global_init(CURL_GLOBAL_ALL);

    curlHandle = curl_easy_init();

    if ( !curlHandle )
        log( LOG_ERROR, "HttpForward::Source: Can't open libcurl" );
}

HttpForward::~HttpForward()
{
    if ( pURL )
        free( (void*)pURL );

    curl_global_cleanup();
}

static size_t write_data(void *ptr, size_t size, size_t nmemb, void *stream)
{
  DBG4( "HttpForward:write_data ptr=%08X, size=%d, nmemb=%d, stream=%08X" , (unsigned int)ptr, size, nmemb, (unsigned)stream );
  #if DEBUG
  fwrite( ptr, size, nmemb, stdout );
  #endif
  return nmemb;
}

int HttpForward::Forward( const char *msg )
{
    if ( curlHandle )
    {
        strcpy( urlBuffer, pURL );
				msg = curl_escape( msg, 0 );
        strcat( urlBuffer, msg );
        DBG1( "HttpForward:Forward %s" , (int) urlBuffer );
        curl_easy_setopt( curlHandle, CURLOPT_URL, urlBuffer );
        curl_easy_setopt( curlHandle, CURLOPT_NOPROGRESS, 1L);
        curl_easy_setopt( curlHandle, CURLOPT_WRITEFUNCTION, write_data );
        curl_easy_setopt( curlHandle, CURLOPT_WRITEHEADER, 1L );
        curl_easy_setopt( curlHandle, CURLOPT_WRITEDATA, 2L );
        CURLcode res = curl_easy_perform( curlHandle );
        DBG1( "HttpForward:Forward res=%d" , (int)res );
				curl_free( (void*)msg );
    }

    return 0;
}
