#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <errno.h>
#include <time.h>

#include <curl/curl.h>


#include "tcpendpoint.h"
#include "httpforward.h"
#include "logger.h"


#define FORWARD_URL     "http://myserver.com/test/test.php?msg="


TcpEndPoint::TcpEndPoint( int _sock, struct sockaddr_in _addr )
{
    sock = _sock;
    addr = _addr;

    httpForward = new HttpForward( FORWARD_URL );

    DBG2( "TcpEndPoint::TcpEndPoint %04u @ 0x%08X", nIdx, (int)this );
}

TcpEndPoint::~TcpEndPoint()
{
    DBG2( "TcpEndPoint::~TcpEndPoint %04u @ 0x%08X", nIdx, (int)this );
    if ( sock > 0 )
        close( sock );
}

int TcpEndPoint::SetTcpKeepAlive( int _idle, int _interval, int _count )
{
    int ret = 0;

    if ( sock > 0 )
    {
        int optval = 0;
        size_t optlen = sizeof optval;

        if ( _idle > 0 )
        {
            optval = 1;
            ret = setsockopt( sock, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen);
            if ( ret < 0 )
                DBG2( "KeepAlive error SO_KEEPALIVE [%d:%s]", errno, (int)strerror(errno) );
            optval = _idle;
            ret = setsockopt( sock, SOL_TCP, TCP_KEEPIDLE, &optval, optlen);
            if ( ret < 0 )
                DBG2( "KeepAlive error TCP_KEEPIDLE [%d:%s]", errno, (int)strerror(errno) );
            optval = _interval;
            ret = setsockopt( sock, SOL_TCP, TCP_KEEPINTVL, &optval, optlen);
            if ( ret < 0 )
                DBG2( "KeepAlive error TCP_KEEPINTVL [%d:%s]", errno, (int)strerror(errno) );
            optval = _count;
            ret = setsockopt( sock, SOL_TCP, TCP_KEEPCNT, &optval, optlen);
            if ( ret < 0 )
                DBG2( "KeepAlive error TCP_KEEPCNT [%d:%s]", errno, (int)strerror(errno) );
        }
        else
        {
            optval = 0;
            ret = setsockopt( sock, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen);
            if ( ret < 0 )
                DBG2( "KeepAlive error SO_KEEPALIVE [%d:%s]", errno, (int)strerror(errno) );
        }

    }

    return ret;
}

int TcpEndPoint::Read( void )
{
    int nbytes;

    bzero( buffer, BUFF_LEN );

    nbytes = recv( sock, buffer, BUFF_LEN, 0 );

    DBG2( "TcpEndPoint::Read %d bytes = '%s'", nbytes, (int)buffer );

    if ( nbytes <= 0 )
    {
        log( LOG_ERROR, "error on receiving from TcpEndPoint [%d,%d,%s]", nbytes, errno, (int)strerror(errno) );
    }
    else if ( (nbytes >= 15) && isdigit(buffer[0]) && isdigit(buffer[14]) && (buffer[15]==';') )
    {
        // SPECIAL FOR COBAN : This is IMEI number, we shall answer with "ON"
        Send( "ON", 2 );
    }
    else if ( (nbytes >= 24) && (0 == strncmp( (char*)buffer, "##,imei:", 8 )) && isdigit(buffer[0+8]) && isdigit(buffer[14+8]) && (buffer[15+8]==',') )
    {
        // SPECIAL FOR COBAN : This is another Hearbeat, we shall answer with "LOAD"
        Send( "LOAD", 4 );
    }
    else
    {
        httpForward->Forward( (char*)buffer );
    }

    return nbytes;
}

int TcpEndPoint::Send( const unsigned char *_pbuff, int _nbytes )
{
    int ret = _nbytes;

    while ( _nbytes > 0 )
    {
        ret = send( sock, _pbuff, _nbytes, 0 );
        if ( ret <= 0 )
        {
            log( LOG_ERROR, "can't send to TcpEndPoint [%d:%s]", errno, (int)strerror(errno) );
            break;
        }
        _nbytes -= ret;
        _pbuff += ret;
    }

    return ret;
}

